
import React from 'react';

import {
    Image,
    View,
    StyleSheet,
} from 'react-native';

import {
    ViewStylePropTypes,
} from './RNExt';

import {
    WMView,
} from 'wm-react-native-components';

import {
    BaseViewComponent
} from 'wm-react-native-base';

/**
 * image必须存在的 style 属性
 * @type {string[]}
 */
const IMAGE_NEED_PROPS_STYLE = ['width', 'height', 'borderRadius'];

/**
 * 支持 defaultSource
 * <WMImage style={styles.commentImage} defaultSource={require('../images/icon_default_head.png')} source={{uri:data.pictureUrl}} />
 */
export default class WMImage extends BaseViewComponent{

    /**
     *
     * @type {null}
     */
    viewStyle = null;

    /**
     *
     * @type {null}
     */
    imgStyle = null;

    constructor(props){
        super(props);
        this.state = {source:this.props.source, defaultSource:this.props.defaultSource};

        const { style } = this.props;

        this._dealStyle(this.viewStyle = {}, this.imgStyle = {}, style);
    }

    componentWillReceiveProps(nextProps){
        if(nextProps){
            let { defaultSource, source, style } = nextProps;
            if(style){
                this._dealStyle(this.viewStyle = {}, this.imgStyle = {}, style);
            }
            this.setState({...nextProps});
        }
    }

    /**
     * 处理 props style 拆分成 viewStyle 和 imageStyle
     */
    _dealStyle = (targetViewStyle, targetImgStyle, propsStyle)=>{
        if(!targetViewStyle || !targetImgStyle){
            return ;
        }
        if(Array.isArray(propsStyle)){
            for(let index = 0; index<propsStyle.length; index++){
                let pStyle = propsStyle[index];
                if(typeof pStyle === 'number'){
                    pStyle = StyleSheet.flatten(pStyle);
                }
                for(let key in pStyle){
                    if(ViewStylePropTypes.hasOwnProperty(key)){
                        targetViewStyle[key] = pStyle[key];

                        if(IMAGE_NEED_PROPS_STYLE.contains(key)){
                            targetImgStyle[key] = propsStyle[key];
                        }
                    }else{
                        targetImgStyle[key] = pStyle[key];
                    }
                }
            }
        }else{
            if(typeof propsStyle === 'number'){
                propsStyle = StyleSheet.flatten(propsStyle);
            }
            for(let key in propsStyle){
                if(ViewStylePropTypes.hasOwnProperty(key)){
                    targetViewStyle[key] = propsStyle[key];

                    if(IMAGE_NEED_PROPS_STYLE.contains(key)){
                        targetImgStyle[key] = propsStyle[key];
                    }
                }else{
                    targetImgStyle[key] = propsStyle[key];
                }
            }
        }

        this._correctWidthHeight();

        if(this.viewStyle.borderWidth && this.viewStyle.borderWidth > 0){
            this.viewStyle.width +=this.viewStyle.borderWidth*2;
            this.viewStyle.height +=this.viewStyle.borderWidth*2;
        }
    }

    /**
     * style中没有设置 width & height 尝试从 source || defaultSource 中获取
     * @private
     */
    _correctWidthHeight = ()=>{
        let { source, defaultSource } = this.state;

        //style中没有设置 width & height 尝试从 source || defaultSource 中获取
        if(!this.viewStyle.width || this.viewStyle.width<=0){
            if(source && typeof source === 'object' && source.width>0){
                this.viewStyle.width = source.width;
            }else if(defaultSource && typeof defaultSource === 'object' && defaultSource.width>0){
                this.viewStyle.width = defaultSource.width;
            }
            this.imgStyle.width = this.viewStyle.width;
        }

        //style中没有设置 width & height 尝试从 source || defaultSource 中获取
        if(!this.viewStyle.height || this.viewStyle.height<=0){
            if(source && typeof source === 'object' && source.height>0){
                this.viewStyle.height = source.height;
            }else if(defaultSource && typeof defaultSource === 'object' && defaultSource.height>0){
                this.viewStyle.height = defaultSource.height;
            }
            this.imgStyle.height = this.viewStyle.height;
        }
    }

    /**
     * 设置url
     * @param url
     */
    setUrl = (url, callback)=>{
        if(url){
            this.setState({source:{uri:url}}, callback);
        }
    }

    /**
     * 获取当前 url
     */
    getUrl = ()=>{
        if(this.state.source && this.state.source.uri){
            return this.state.source.uri;
        }
    }

    /**
     * 设置source
     */
    setSource = (source)=>{
        this.setState({source:source});
    }

    /**
     * 设置 defaultSource
     * @param defaultSource
     */
    setDefaultSource = (defaultSource)=>{
        this.setState({defaultSource: defaultSource});
    }

    render(){
        const { style, ...otherProps } = this.props;

        // console.log('this.state.source=====> ', this.state.source);
        return (
            <WMView style={[this.viewStyle, {justifyContent:'center', alignItems:'center', overflow:'hidden'}]}>
                <Image {...otherProps} style={[this.imgStyle, this.viewStyle!=null?{width:this.viewStyle.width, height:this.viewStyle.height}:null]} resizeMode={'cover'} source={this.state.defaultSource}/>
                <Image ref='img' {...otherProps} style={[this.imgStyle, {position:'absolute', left:0, top:0}]} source={this.state.source}/>
            </WMView>
        );
    }

}