
import React from 'react';
import {
    TextInput,
} from 'react-native';


import {
    BaseViewComponent
} from 'wm-react-native-base';

/**
 * 封装Text
 */
export default class WMTextInput extends BaseViewComponent {

    // 构造
    constructor(props) {
        super(props);
        const { isShow } = this.props;
        this.state = {isShow: (isShow === undefined || isShow === null) ? true : isShow, cardExpiry: '', lastCardExpiry: ''};
    }

    _containChineseCharacter = (text) => {
        let reg = /[\u4E00-\u9FA5\uF900-\uFA2D]/;
        return reg.test(text);
    }

    _isNumberCharacter = (text) => {
        let reg = /^[0-9]*$/;
        return reg.test(text);
    }

    _handlingCardExpiry = (text) => {

        //是否是正常的 TextInput 不处理任何逻辑
        const { isNormalInput } = this.props;
        if(isNormalInput){
            this.setState({
                cardExpiry: text
            }, ()=>{
                this.props.onChangeTextInput && this.props.onChangeTextInput(text);
            });
            return ;
        }

        //删除
        if (text.indexOf('.') >= 0 || (this.props.noneChinese && this._containChineseCharacter(text))) {
            // Since the keyboard will have a decimal and we don't want
            // to let the user use decimals, just exit if they add a decimal
            // Also, we only want 'MM/YY' so if they try to add more than
            // 5 characters, we want to exit as well
            return;
        }

        //如果是数字键盘，只能输入数字
        let trimText = text.replace(/ /g, '');
        if(this.props.keyboardType && this.props.keyboardType === 'numeric' && !this._isNumberCharacter(trimText)){
            return;
        }

        if (this.state.lastCardExpiry.length > text.length) {
            text = text.RTrim();
        } else {
            if (!this.props.isTelInput) {
                if (this.props.spaceLength >= 1 && text.length > 0 && text.length % (this.props.spaceLength + 1) === 0) {
                    // This is where the user has typed 2 numbers so far
                    // We can manually add a slash onto the end
                    // We check to make sure the current value was only 1 character
                    // long so that if they are backspacing, we don't add on the slash again
                    text = text.splice(text.length - 1, 0, ' ');
                }
            } else {
                if (text.length <= 4) {
                    if (text.length % 4 == 0) {
                        text = text.splice(text.length - 1, 0, ' ');
                    }
                } else {
                    let length = text.length - 4;
                    if (length > 0 && length % 5 == 0) {
                        text = text.splice(text.length - 1, 0, ' ');
                    }
                }
            }
        }

        this.state.lastCardExpiry = text;
        // Update the state, which in turns updates the value in the text field
        this.setState({
            cardExpiry: text
        }, ()=>{
            this.props.onChangeTextInput && this.props.onChangeTextInput(text);
        });
    }

    getText = () => {
        //说明需要加入空格，则返回应该去掉空格
        if (this.props.isTelInput || this.props.spaceLength >= 1) {
            return this.state.cardExpiry.replace(/ /g, '');
        } else {
            return this.state.cardExpiry;
        }
    }

    setText = (text) => {
        this.setState({
            cardExpiry: text
        });
    }

    getTrimText = () => {
        return this.state.cardExpiry.replace(/ /g, '');
    }

    getText = () => {
        return this.state.cardExpiry;
    }

    setTelText = (value) => {
        if (value && typeof (value) === 'string' && value.length === 11) {
            let spaceTel = value.substring(0, 3) + " " + value.substring(3, 7) + " " + value.substring(7, 11);
            this._handlingCardExpiry(spaceTel);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props)=>{
        this.refs.input && this.refs.input.setNativeProps(props);
    }

    /**
     * 获取焦点
     */
    focus = ()=>{
        this.refs.input && this.refs.input.focus();
    }

    /**
     * 清除焦点
     */
    clearFocus = ()=>{
        this.refs.input && this.refs.input.blur && this.refs.input && this.refs.input.blur();
    }

    /**
     * 是否有焦点
     * @returns {*|boolean}
     */
    isFocused = ()=>{
        return this.refs.input && this.refs.input.isFocused();
    }

    /**
     * 当前是否已经显示
     */
    isShow = ()=>{
        return this.state.isShow;
    }

    /**
     * 隐藏
     */
    hidden = ()=>{
        this.setState({isShow:false});
    }

    /**
     * 显示
     */
    show = ()=>{
        this.setState({isShow:true});
    }

    render() {
        if(!this.state.isShow){
            return null;
        }
        return (
            <TextInput
                spaceLength={0}
                onChangeText={this._handlingCardExpiry}
                placeholderTextColor='#CBCBCB'
                {...this.props}
                value={this.state.cardExpiry}
                underlineColorAndroid='transparent'
                ref='input'
            />
        );
    }
}