import React, { Component } from 'react';

import PropTypes from 'prop-types';

import {
	StyleSheet,
	TouchableHighlight,
	TouchableWithoutFeedback,
	Image,
	View,
    Dimensions,
} from 'react-native';

const { width:SCREEN_WIDTH, height:SCREEN_HEIGHT } = Dimensions.get('window');

import Carousel from './Carousel';

export default class BannerView extends Component {

	static propTypes = {
        dataList:PropTypes.arrayOf(PropTypes.shape({    //数据
            imgUrl:PropTypes.string.isRequired, //图片
            targetObj:PropTypes.object    //每个item对应的自定义数据
        })),
	}

    static defaultProps = {
        dataList:[],
    }

	constructor(props) {
	  super(props);
	
	  this.state = {
	  	dataList: this.props.dataList
	  };
	}

	render() {
		const { dataList } = this.state;

		if(dataList.length > 0){
            let height = SCREEN_WIDTH * 0.5;
            let width = SCREEN_WIDTH;

            return (
				<View>
					<Carousel
						delay={5000}
						style={{height: height}}
						pageInfoBottomContainerStyle={{bottom: 10}}
						activeDotStyle={{backgroundColor: '#ffffff'}}
						autoplay
						pageInfo={true}
                        {...this.props}
					>
                        {
                            dataList.map((item, i) => {
                                return (
									<TouchableWithoutFeedback
										key={i}
										onPress={() => this.props.onPress && this.props.onPress(i)}
										underlayColor='rgba(0,0,0,0)'
									>
										<Image style={{marginTop:0, height:height, width: width}}
											   source={{uri: item.imgUrl}}
										>
										</Image>
									</TouchableWithoutFeedback>
                                )
                            })
                        }
					</Carousel>
				</View>
            );
		}else{
			return null;
		}
	}
}

const styles = StyleSheet.create({
	activeDot: {
		backgroundColor: '#FFFFFF', 
		width: 8, 
		height: 8, 
		borderRadius: 4, 
		marginLeft: 3, 
		marginRight: 3, 
		marginTop: 0, 
		marginBottom: 0
	},
	dot: {
		backgroundColor: '#000000', 
		opacity: 0.2,
		width: 8, 
		height: 8, 
		borderRadius: 4, 
		marginLeft: 3, 
		marginRight: 3, 
		marginTop: 0, 
		marginBottom: 0
	},
});