/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2018/5/23
 * Time: 14:17
 */

import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';

import {
    ART,
    View,
    Animated,
    StyleSheet,
} from 'react-native';

const {
    Surface,
    Path,
    Shape,
} = ART;

import {
    StyleSheetPropType,
    ViewStylePropTypes,
} from '../RNExt';

const viewStylePropType = StyleSheetPropType(ViewStylePropTypes);

/**
 * 动画时间
 * @type {number}
 */
const DURATION = 180;

/**
 * 直线 进度条
 */
class LineProgressBar extends PureComponent {

    /**
     *
     * @type {{width: React$PropType$Primitive.<number>, height: React$PropType$Primitive.<number>, progress: React$PropType$Primitive.<number>}}
     */
    static propTypes = {
        style:viewStylePropType,    //样式
        progress:PropTypes.number,   //进度
    };

    /**
     *
     * @type {{width: number, height: number}}
     */
    static defaultProps = {
        style:{
            width:200,
            height:10,
        },
        progress: 0,
    }

    /**
     *
     * @type {number}
     * @private
     */
    _width = 0;
    /**
     *
     * @type {number}
     * @private
     */
    _height = 0;

    /**
     *
     * @type {number}
     * @private
     */
    _startX = 0;

    /**
     * 背景 path
     * @type {null}
     * @private
     */
    _bgPath = null;

    constructor(props) {
        super(props);

        this._getSize();

        const { progress } = this.props;

        this._startX = this._height / 2 ;

        this._bgPath = Path();
        this._bgPath.moveTo(this._startX, this._height / 2);
        this._bgPath.lineTo(this._width - this._startX, this._height / 2);

        this.state = {
            progress: progress,
            progressPath: this._calProgressPath(progress),
        };
    }

    componentWillReceiveProps(nextProps) {
        const {progress} = nextProps;

        if (progress !== this.state.progress) {
            this.setProgress(progress);
        }
    }

    /**
     * 获取 宽度、高度
     */
    _getSize = ()=>{
        let { style }  = this.props;

        if(style){
            if(Array.isArray(style)){
                for(let index = 0; index<style.length; index++){
                    let pStyle = style[index];
                    if(typeof pStyle === 'number'){
                        pStyle = StyleSheet.flatten(pStyle);
                    }
                    if(pStyle.width){
                        this._width = pStyle.width;
                    }
                    if(pStyle.height){
                        this._height = pStyle.height;
                    }
                }
            }else{
                if(typeof style === 'number'){
                    style = StyleSheet.flatten(style);
                }
                if(style.width){
                    this._width = style.width;
                }
                if(style.height){
                    this._height = style.height;
                }
            }
        }
        if(typeof this._width !== 'number'){
            throw new Error('LineProgressBar 宽度请设置数字');
        }
        if(typeof this._height !== 'number'){
            throw new Error('LineProgressBar 高度请设置数字');
        }
        if(this._width <= 0){
            throw new Error('LineProgressBar 请设置宽度');
        }
        if(this._height <= 0){
            throw new Error('LineProgressBar 请设置高度');
        }
    }

    /**
     * 通过 progress 计算 path
     * @private
     */
    _calProgressPath = (progress) => {
        if(progress === 0){
            return null;
        }
        const progressPath = Path();
        progressPath.moveTo(this._startX, this._height / 2);

        let toX = (this._width - this._startX) * progress || this._startX;
        progressPath.lineTo(toX, this._height / 2);

        return progressPath;
    }

    /**
     * 设置进度
     * @param progress
     */
    setProgress = (progress: number) => {
        if (progress > 1 || progress === this.state.progress) {
            return;
        }
        this.setState({
            progress: progress,
            progressPath: this._calProgressPath(progress),
        });
    }

    /**
     *
     */
    getProgress = () => {
        return this.state.progress;
    }

    render() {
        const { style } = this.props;

        return (
            <View style={style} pointerEvents={'none'}>
                <Surface
                    width={this._width}
                    height={this._height}
                >

                    <Shape d={this._bgPath} stroke='#E8E8E9' strokeWidth={this._height}/>

                    <Shape d={this.state.progressPath} stroke='#0075FF' strokeWidth={this._height}/>

                </Surface>
            </View>
        );
    }

}

/**
 *
 */
const LineProgressBarAnimTemp = Animated.createAnimatedComponent(LineProgressBar);

/**
 *
 */
export default class LineProgressBarAnim extends PureComponent {

    constructor(props) {
        super(props);

        this.state = {
            progressAnim: new Animated.Value(0),
        }
    }

    componentWillUnmount() {
        this.state.progressAnim.stopAnimation();
    }

    /**
     * 设置进度
     * @param progress
     */
    setProgress = (progress: number) => {
        if (progress > 1) {
            return;
        }
        Animated.timing(this.state.progressAnim, {
            toValue: progress,
            duration: DURATION,
            isInteraction: false,
        }).start();
    }

    /**
     *
     * @returns {*}
     */
    getProgress = () => {
        return this.refs.lineProgressBarAnimTemp && this.refs.lineProgressBarAnimTemp._component.getProgress();
    }

    render() {
        return (
            <LineProgressBarAnimTemp
                key={''}
                ref={'lineProgressBarAnimTemp'}
                {...this.props}
                progress={this.state.progressAnim}
            />
        );
    }

}